;------------------------------------
;
;	SDISK II LCD Farmware (3 of 3)
;	
;	2010.11.11 by Koichi Nishida
;
;------------------------------------

;
; LCD unit controller
;
; use ATTINY2313 AVR, internal 4MHz OSC
;  fuse low: 10100010
;      high: 11-11111
; 
; hardware information:
; Port A
;   bit 0: DSP
;   bit 1: DIN
; Port B
;   bit 5-0: I
; Port D
;   bit 0: RX
;   bit 1: TX
;   bit 2: CLK = CKOUT
;   bit 5: RST
;

/*
This is a part of the firmware for DISK II emulator by Nishida Radio.

Copyright (C) 2010 Koichi NISHIDA
email to Koichi NISHIDA: tulip-house@msf.biglobe.ne.jp

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

.include "tn2313def.inc"

.cseg
	rjmp	VECT_RESET
	rjmp	VECT_INT0
	rjmp	VECT_INT1
	rjmp	VECT_TIMER1_CAPT
	rjmp	VECT_TIMER1_COMPA
	rjmp	VECT_TIMER1_OVF
	rjmp	VECT_TIMER0_OVF
	rjmp	VECT_USART0_RX
	rjmp	VECT_USART0_UDRE
	rjmp	VECT_USART0_TX
	rjmp	VECT_ANALOG_COMP
	rjmp	VECT_PCINT
	rjmp	VECT_TIMER1_COMPB
	rjmp	VECT_TIMER0_COMPA
	rjmp	VECT_TIMER0_COMPB
	rjmp	VECT_USI_START
	rjmp	VECT_USI_OVERFLOW
	rjmp	VECT_EE_READY
	rjmp	VECT_WDT_OVERFLOW
VECT_INT0:
VECT_INT1:
VECT_TIMER1_CAPT:
VECT_TIMER1_COMPA:
VECT_TIMER1_OVF:
VECT_TIMER0_OVF:
;VECT_USART0_RX:
VECT_USART0_UDRE:
VECT_USART0_TX:
VECT_ANALOG_COMP:
VECT_PCINT:
VECT_TIMER1_COMPB:
VECT_TIMER0_COMPA:
VECT_TIMER0_COMPB:
VECT_USI_START:
VECT_USI_OVERFLOW:
VECT_EE_READY:
VECT_WDT_OVERFLOW:
	reti

VECT_USART0_RX:
	push	r18
	in		r18,SREG
	push	r18
USART_Receive:
	sbis	UCSRA, RXC
	rjmp	USART_Receive
	in		r18, UDR
	out		PORTB,r18
	cpi		r18,0xff
	brne	VU_CONT
	ldi		r18,0b00000001	; DSP = 1
	out		PORTA,r18
	pop		r18
	out		SREG,r18
	pop		r18
	reti
VU_CONT:
	; display a character
	ldi		r18,0b00000010	; DIN = 1
	out		PORTA,r18
	nop
	nop
	ldi		r18,0b00000000	; DIN = 0
	out		PORTA,r18
	pop		r18
	out		SREG,r18
	pop		r18
	reti

VECT_RESET:
	cli

	; 500KHz at internal 4MHz osc
	ldi		r18,0b10000000
	out		CLKPR,r18
	ldi		r18,0b00000011
	out		CLKPR,r18

	ldi		r18,0b00000011
	out		DDRA,r18
	ldi		r18,0b01111111
	out		DDRB,r18
	ldi		r18,0b00100110
	out		DDRD,r18

	ldi		r18,0b00000000	; DIN = DSP = 0
	out		PORTA,r18

	ldi		r18, LOW(RAMEND)
	out		SPL, r18

	// 4800 baud at 500KHz
	// 8data, 2stop bit
	ldi		r18,6
	out		UBRRL,r18
	ldi		r18,0
	out		UBRRH,r18
	ldi 	r18, (1<<RXEN)|(1<<RXCIE)
	out		UCSRB,r18
	ldi 	r18, (1<<USBS)|(3<<UCSZ0)
	out 	UCSRC,r18

	; reset LCD
	ldi		r18,0b00000000
	out		PORTD,r18
	ldi		r18,0b00100000
	out		PORTD,r18
	nop
	nop
	ldi		r18,0b00000000
	out		PORTD,r18

	sei
LOOP:
	rjmp	LOOP


